<!DOCTYPE html>

{% extends "layout.html.jinja" %}

<h1>{{_('Errored Tasks')}} - {{device.device_type}}</h1>

{%block content %}
<table class="table table-bordered table-striped ">
  <thead>
    <tr>
      <th>{{_('Create Date')}}</th>
      <th>{{_('UUID')}}</th>
      <th>{{_('Data')}}</th>
    </tr>
  </thead>

  <tbody>
    {% for task in device.errored_tasks %}
    <tr>
      <td>{{task.create_date}}</td>
      <td>{{task.uuid}}</td>
      <td>{{task.data}}</td>
    </tr>
    {% endfor %}
  </tbody>
</table>
{% endblock %}
