<!DOCTYPE html>

{% extends "layout.html.jinja" %}

<h1>{{_('Disconnections')}} - {{device.device_type}}</h1>

{%block content %}
<table class="table table-bordered table-striped ">
  <thead>
    <tr>
      <th>{{_('Date')}}</th>
      <th>{{_('Error')}}</th>
    </tr>
  </thead>

  <tbody>
    {% for disconnection in device.disconnections %}
    <tr>
      <td>{{disconnection["date"]}}</td>
      <td>{{disconnection["error"]}}</td>
    </tr>
    {% endfor %}
  </tbody>
</table>
{% endblock %}
